'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.
'''

import os
import pygame
from pygame.locals import *

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '..', 'data'))

def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(filepath(filename), mode)

images = {}
def load_image(filename, colorkey = -1):
    try:
        image = images[filename]
    except:
        fullname = filepath(filename)

        try:
            image = pygame.image.load(fullname)
        except:
            print "Cannot load image: ", fullname
            raise SystemExit
        
        image = image.convert()
        if colorkey == -1:
            colorkey = image.get_at((0,0))
                
        image.set_colorkey(colorkey, RLEACCEL)
        images[filename] = image
    return image, image.get_rect()

def load_font(filename, size):
    return pygame.font.Font(filepath(filename), size)
    