import data
import pygame
from pygame.locals import *
from vector import Vector
import playerstates

class Player(pygame.sprite.DirtySprite):
    def __init__(self, position = Vector(0.0, 0.0)):
        pygame.sprite.DirtySprite.__init__(self)
        self.position = position
        self.velocity = Vector(0.0,0.0)
        self.dirty = 2
        self.alive = True
        self.lives = 3
        
        self.states = {"hanging": playerstates.State_Hanging(), \
            "sliding_down": playerstates.State_SlidingDown(), \
            "hanging_and_looking_up": playerstates.State_HangingAndLookingUp(), \
            "hanging_and_looking_left": playerstates.State_HangingAndLookingLeft(), \
            "hanging_and_looking_right": playerstates.State_HangingAndLookingRight(), \
            "jump_left": playerstates.State_JumpLeft(), \
            "jump_right": playerstates.State_JumpRight(), \
            "jump_and_fly_up": playerstates.State_JumpAndFlyUp(), \
            "flying_left": playerstates.State_FlyingLeft(), \
            "flying_right": playerstates.State_FlyingRight(), \
            "grab_left": playerstates.State_GrabLeft(), \
            "grab_right": playerstates.State_GrabRight(), \
            "swinging_left": playerstates.State_SwingingLeft(), \
            "swinging_right": playerstates.State_SwingingRight(), \
            "dead": playerstates.State_Dead(), }
        
        self.currentstate = self.states["hanging"]
        self.colliderect = pygame.Rect((self.position.x, self.position.y),(40, 80))
        
    def handle_input(self, world, keystates, previous_keystates):
        self.currentstate.input(self, world, keystates, previous_keystates)
    
    def update(self, dt, world):
        self.currentstate.update(self, world, dt)
        self.rect = self.currentstate.image_rect
        self.rect.center = (self.position.x, self.position.y)
        self.colliderect.center = (self.position.x, self.position.y)
        self.image = self.currentstate.image
    
    def change_state(self, world, nextstate):
        self.currentstate.exit(self, world)
        self.currentstate = nextstate
        self.currentstate.enter(self, world)
        
    def add_life(self, world):
        if self.lives < world.max_lives:
            self.lives += 1
            
    def set_y(self, new_y):
        self.position.y = new_y
        self.colliderect.centery = new_y
        self.rect.centery = new_y
    