from config import KeyConfigs
import data
from vector import Vector
from helpers import key_pressed, key_released, key_changed_to_pressed, key_changed_to_released, key_held_down
import math
import sound

JUMP_SPEED_X = 0.2
JUMP_SPEED_Y = 0.05
JUMP_SPEED_UP = 0.4
SLIDE_SPEED = 0.3
CLIMB_SPEED = -0.05
GRAB_DISTANCE = 5
GRAB_DISTANCE_SQ = GRAB_DISTANCE ** 2
GRAVITY = Vector(0.0, 0.0001)
JUMP_UP_GRAVITY = Vector(0.0, 0.0005)
TREE_DISTANCE = 110

class State_Basic:
    def __init__(self):
        pass
    
    def update(self, player, world, dt):
        pass

    def enter(self, player, world):
        pass
    
    def exit(self, player, world):
        pass

    def input(self, player, world, keystates, previous_keystates):
        pass


class State_Hanging(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_hanging.PNG")
    
    def enter(self, player, world):
        player.position.x = world.rope.x
        player.velocity = Vector(0.0, 0.0)
    
    def input(self, player, world, keystates, previous_keystates):
        if key_pressed(keystates, KeyConfigs.KEY_UP):
            player.change_state(world, player.states["hanging_and_looking_up"])
        elif key_pressed(keystates, KeyConfigs.KEY_LEFT):
            player.change_state(world, player.states["hanging_and_looking_left"])
        elif key_pressed(keystates, KeyConfigs.KEY_RIGHT):
            player.change_state(world, player.states["hanging_and_looking_right"])
        elif key_pressed(keystates, KeyConfigs.KEY_DOWN):
            player.change_state(world, player.states["sliding_down"])
        elif key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_JUMP):
            player.change_state(world, player.states["jump_and_fly_up"])
            
class State_SlidingDown(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_sliding_down.PNG")
        
    def update(self, player, world, dt):
        player.position += Vector(0.0, SLIDE_SPEED) * dt
        if player.position.y > world.height:
            player.position.y = world.height
        
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_released(keystates, previous_keystates, KeyConfigs.KEY_DOWN):
            player.change_state(world, player.states["hanging"])


class State_HangingAndLookingUp(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_hanging_and_looking_up.PNG")
        
    def update(self, player, world, dt):
        player.position += Vector(0.0, CLIMB_SPEED) * dt
    
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_released(keystates, previous_keystates, KeyConfigs.KEY_UP):
            player.change_state(world, player.states["hanging"])
        elif key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_JUMP):
            player.change_state(world, player.states["jump_and_fly_up"])


class State_HangingAndLookingLeft(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_hanging_and_looking_left.PNG")

    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_released(keystates, previous_keystates, KeyConfigs.KEY_LEFT):
            player.change_state(world, player.states["hanging"])
        elif key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_JUMP):
            player.change_state(world, player.states["jump_left"])


class State_HangingAndLookingRight(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_hanging_and_looking_right.PNG")

    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_released(keystates, previous_keystates, KeyConfigs.KEY_RIGHT):
            player.change_state(world, player.states["hanging"])
        elif key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_JUMP):
            player.change_state(world, player.states["jump_right"])
    
class State_JumpLeft(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_jumping_left.PNG")

    def enter(self, player, world):
        player.velocity = Vector(-JUMP_SPEED_X, -JUMP_SPEED_Y)
    
    def update(self, player, world, dt):
        player.change_state(world, player.states["flying_left"])
        
    
class State_JumpRight(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_jumping_right.PNG")

    def enter(self, player, world):
        player.velocity = Vector(JUMP_SPEED_X, -JUMP_SPEED_Y)
    
    def update(self, player, world, dt):
        player.change_state(world, player.states["flying_right"])

class State_JumpAndFlyUp(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_jumping_up.PNG")
    
    def enter(self, player, world):
        player.velocity = Vector(0.0, -JUMP_SPEED_UP)
    
    def update(self, player, world, dt):
        player.velocity += JUMP_UP_GRAVITY * dt
        player.position += player.velocity * dt
        
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_GRAB):
            player.change_state(world, player.states["hanging"])
            if key_pressed(keystates, KeyConfigs.KEY_UP):
                player.change_state(world, player.states["hanging_and_looking_up"])
    
class State_FlyingLeft(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_jumping_left.PNG")
    
    def update(self, player, world, dt):
        player.velocity += GRAVITY * dt
        player.position += player.velocity * dt
        if player.position.y > world.height or player.position.x < TREE_DISTANCE or player.position.x > world.width - TREE_DISTANCE:
            player.change_state(world, player.states["dead"])
    
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_GRAB):
            player.change_state(world, player.states["grab_left"])

  
class State_FlyingRight(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_jumping_right.PNG")
    
    def update(self, player, world, dt):
        player.velocity += GRAVITY * dt
        player.position += player.velocity * dt
        
        if player.position.y > world.height or player.position.x < TREE_DISTANCE or player.position.x > world.width - TREE_DISTANCE:
            player.change_state(world, player.states["dead"])
    
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_GRAB):
            player.change_state(world, player.states["grab_right"])
 
    
class State_GrabLeft(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_grabbing_left.PNG")
    
    def update(self, player, world, dt):
        player.velocity += GRAVITY * dt
        player.position += player.velocity * dt
        
        if player.position.y > world.height or player.position.x < TREE_DISTANCE or player.position.x > world.width - TREE_DISTANCE:
            player.change_state(world, player.states["dead"])
        
        #check for collision with rope
        if player.velocity.x < 0 and math.fabs((player.position.x + player.velocity.x * dt) - world.rope.x) < GRAB_DISTANCE:
            player.change_state(world, player.states["hanging"])
            
        #check for collision with vine
        vinelink, vine = world.get_closest_vinelink(player.position)
        if vine != None and vinelink != None:
            if (vinelink.position - player.position).magnitude_sq() < GRAB_DISTANCE_SQ:
                attach_link = vine.catch(vinelink, player)
                player.change_state(world, player.states["swinging_left"])

    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_released(keystates, previous_keystates, KeyConfigs.KEY_GRAB):
            player.change_state(world, player.states["flying_left"])
    
    
class State_GrabRight(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_grabbing_right.PNG")
    
    def update(self, player, world, dt):
        player.velocity += GRAVITY * dt
        player.position += player.velocity * dt
        
        if player.position.y > world.height or player.position.x < TREE_DISTANCE or player.position.x > world.width - TREE_DISTANCE:
            player.change_state(world, player.states["dead"])
        
        #check for collision with rope
        if 0 < player.velocity.x and math.fabs((player.position.x + player.velocity.x * dt) - world.rope.x) < GRAB_DISTANCE:
            player.change_state(world, player.states["hanging"])
            
        #check for collision with vine
        vinelink, vine = world.get_closest_vinelink(player.position)
        if vine != None and vinelink != None:
            if (vinelink.position - player.position).magnitude_sq() < GRAB_DISTANCE_SQ:
                attach_link = vine.catch(vinelink, player)
                player.change_state(world, player.states["swinging_right"])

    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_released(keystates, previous_keystates, KeyConfigs.KEY_GRAB):
            player.change_state(world, player.states["flying_right"])
            
class State_SwingingLeft(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_swinging_left.PNG")
        self.attached_vine = None
        self.attached_vinelink = None
    
    def enter(self, player, world):
        sound.play_sound("left.wav")
    
    def update(self, player, world, dt):
        player.position = self.attached_vinelink.position   
        if self.attached_vinelink.velocity.x >= 0:
            player.change_state(world, player.states["swinging_right"])
    
    def exit(self, player, world):
        pass
        
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_JUMP):
            self.attached_vine.let_go(player)
            player.change_state(world, player.states["jump_left"])
            
class State_SwingingRight(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_swinging_right.PNG")
        self.attached_vinelink = None
        self.attached_vine = None
        
    def enter(self, player, world):
        sound.play_sound("right.wav")
        
    def update(self, player, world, dt):
        player.position = self.attached_vinelink.position
        if self.attached_vinelink.velocity.x < 0:
            player.change_state(world, player.states["swinging_left"])
            
    def exit(self, player, world):
        pass
    
    def input(self, player, world, keystates, previous_keystates):
        if key_changed_to_pressed(keystates, previous_keystates, KeyConfigs.KEY_JUMP):
            self.attached_vine.let_go(player)
            player.change_state(world, player.states["jump_right"])
            
class State_Dead(State_Basic):
    def __init__(self):
        State_Basic.__init__(self)
        self.image, self.image_rect = data.load_image("player_dead.PNG")
    
    def enter(self, player, world):
        sound.play_sound("hit.wav");
        player.alive = False
        player.velocity = Vector(0.0, 0.0)
        
    def update(self, player, world, dt):
        player.velocity += GRAVITY * dt
        player.position += player.velocity * dt
        
        if player.position.y > world.height:
            player.change_state(world, player.states["hanging"])
    
    def exit(self, player, world):
        player.lives -= 1
        player.alive = True
        world.reset(world.height)
        if(player.lives == -1):
            world.open_Deathmenu()